@@ -547,7 +547,8 @@ static int fusectl_dump(struct mount_info *pm)
 		}
 
 		for (it = mntinfo; it; it = it->next) {
-			if (it->fstype->code == FSTYPE__FUSE && id == kdev_minor(it->s_dev) && !it->external) {
+			if (it->fstype->code == FSTYPE__FUSE && id == kdev_minor(it->s_dev) &&
+			    !mnt_is_external_bind(it)) {
 				pr_err("%s is a fuse mount but not external\n", it->ns_mountpoint);
 				goto out;
 			}
