@@ -16,34 +16,11 @@
 
 #include "xmalloc.h"
 #include "kfd_ioctl.h"
+#include "amdgpu_plugin_util.h"
 #include "amdgpu_plugin_topology.h"
 
 #define TOPOLOGY_PATH "/sys/class/kfd/kfd/topology/nodes/"
-
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE 1
-#endif
-
-#ifdef COMPILE_TESTS
-#undef pr_err
-#define pr_err(format, arg...) fprintf(stdout, "%s:%d ERROR:" format, __FILE__, __LINE__, ##arg)
-#undef pr_info
-#define pr_info(format, arg...) fprintf(stdout, "%s:%d INFO:" format, __FILE__, __LINE__, ##arg)
-#undef pr_debug
-#define pr_debug(format, arg...) fprintf(stdout, "%s:%d DBG:" format, __FILE__, __LINE__, ##arg)
-
-#undef pr_perror
-#define pr_perror(format, arg...) \
-	fprintf(stdout, "%s:%d: " format " (errno = %d (%s))\n", __FILE__, __LINE__, ##arg, errno, strerror(errno))
-#endif
-
-#ifdef DEBUG
-#define plugin_log_msg(fmt, ...) pr_debug(fmt, ##__VA_ARGS__)
-#else
-#define plugin_log_msg(fmt, ...) \
-	{                        \
-	}
-#endif
+#define MAX_PARAMETER_LEN 64
 
 /* User override options */
 /* Skip firmware version check */
@@ -441,7 +418,9 @@ struct tp_node *sys_add_node(struct tp_system *sys, uint32_t id, uint32_t gpu_id
 
 static bool get_prop(char *line, char *name, uint64_t *value)
 {
-	if (sscanf(line, " %29s %lu", name, value) != 2)
+	char format[16];
+	sprintf(format, " %%%ds %%lu", MAX_PARAMETER_LEN);
+	if (sscanf(line, format, name, value) != 2)
 		return false;
 	return true;
 }
@@ -461,7 +440,7 @@ static int parse_topo_node_properties(struct tp_node *dev, const char *dir_path)
 	}
 
 	while (fgets(line, sizeof(line), file)) {
-		char name[30];
+		char name[MAX_PARAMETER_LEN + 1];
 		uint64_t value;
 
 		memset(name, 0, sizeof(name));
@@ -589,7 +568,7 @@ static int parse_topo_node_mem_banks(struct tp_node *node, const char *dir_path)
 			}
 
 			while (fgets(line, sizeof(line), file)) {
-				char name[30];
+				char name[MAX_PARAMETER_LEN + 1];
 				uint64_t value;
 
 				memset(name, 0, sizeof(name));
@@ -678,7 +657,7 @@ static int parse_topo_node_iolinks(struct tp_node *node, const char *dir_path)
 			}
 
 			while (fgets(line, sizeof(line), file)) {
-				char name[30];
+				char name[MAX_PARAMETER_LEN + 1];
 				uint64_t value;
 
 				memset(name, 0, sizeof(name));
@@ -840,6 +819,9 @@ void topology_free(struct tp_system *sys)
 		list_del(&p2pgroup->listm_system);
 		xfree(p2pgroup);
 	}
+
+	/* Update Topology as being freed */
+	sys->parsed = false;
 }
 
 /**
@@ -1063,7 +1045,7 @@ static bool iolink_match(struct tp_iolink *src, struct tp_iolink *dest)
  *
  * Nodes compatibility are determined by:
  * 1. Comparing the node properties
- * 2. Making sure iolink mappings to CPUs would be compabitle with existing iolink mappings in maps
+ * 2. Making sure iolink mappings to CPUs would be compatible with existing iolink mappings in maps
  *
  * If src_node and dest_node are mappable, then map_device will push the new mapping
  * for src_node -> dest_node into new_maps.
@@ -1241,7 +1223,7 @@ static bool map_devices(struct tp_system *src_sys, struct tp_system *dest_sys, s
 				return true;
 			} else {
 				/* We could not map remaining nodes in the list. Add dest node back
-				 * to list and try to map next dest ndoe in list to current src
+				 * to list and try to map next dest node in list to current src
 				 * node.
 				 */
 				pr_debug("Nodes after [0x%04X -> 0x%04X] did not match, "
@@ -1461,3 +1443,15 @@ int set_restore_gpu_maps(struct tp_system *src_sys, struct tp_system *dest_sys,
 
 	return ret;
 }
+
+int topology_gpu_count(struct tp_system *sys)
+{
+	struct tp_node *node;
+	int count = 0;
+
+	list_for_each_entry(node, &sys->nodes, listm_system)
+		if (NODE_IS_GPU(node))
+			count++;
+	return count;
+}
+
