@@ -126,11 +126,25 @@ extern int write_pidfile(int pid);
 /* message helpers */
 extern int test_log_init(const char *outfile, const char *suffix);
 extern int zdtm_seccomp;
-#define pr_err(format, arg...) test_msg("ERR: %s:%d: " format, __FILE__, __LINE__, ##arg)
-#define pr_perror(format, arg...) \
-	test_msg("ERR: %s:%d: " format " (errno = %d (%s))\n", __FILE__, __LINE__, ##arg, errno, strerror(errno))
-#define fail(format, arg...) \
-	test_msg("FAIL: %s:%d: " format " (errno = %d (%s))\n", __FILE__, __LINE__, ##arg, errno, strerror(errno))
+#define pr_err(format, arg...)                                              \
+	({                                                                  \
+		test_msg("ERR: %s:%d: " format, __FILE__, __LINE__, ##arg); \
+		1;                                                          \
+	})
+
+#define pr_perror(format, arg...)                                                                        \
+	({                                                                                               \
+		test_msg("ERR: %s:%d: " format " (errno = %d (%s))\n", __FILE__, __LINE__, ##arg, errno, \
+			 strerror(errno));                                                               \
+		1;                                                                                       \
+	})
+
+#define fail(format, arg...)                                                                              \
+	({                                                                                                \
+		test_msg("FAIL: %s:%d: " format " (errno = %d (%s))\n", __FILE__, __LINE__, ##arg, errno, \
+			 strerror(errno));                                                                \
+		1;                                                                                        \
+	})
 #define skip(format, arg...) test_msg("SKIP: %s:%d: " format "\n", __FILE__, __LINE__, ##arg)
 #define pass()		     test_msg("PASS\n")
 
@@ -202,4 +216,13 @@ static inline void cleanup_closep(void *p)
 		TEMP_FAILURE_RETRY(close(*pp));
 }
 
+extern int write_value(const char *path, const char *value);
+extern int read_value(const char *path, char *value, int size);
+
+#define container_of(ptr, type, member)                            \
+	({                                                         \
+		const typeof(((type *)0)->member) *__mptr = (ptr); \
+		(type *)((char *)__mptr - offsetof(type, member)); \
+	})
+
 #endif /* _VIMITESU_H_ */
